##################################################################### 
# Copyright (C) 2007 Alex Clemesha <clemesha@gmail.com>
# 
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
##################################################################### 
from twisted.cred import portal, checkers, credentials, error as credError
from twisted.internet import defer
from zope.interface import implements
from knoboo.external.twisted.web2 import iweb

#http resources
from knoboo.resources import resources

#kernel connection
from knoboo.kernel.kernel_manager import kernel

class LoginSystem(object):
    implements(portal.IRealm)

    def __init__(self, dbConnection):
        self.dbConnection = dbConnection
        self.kernels = {} #logged in users kernel connections.
        self.logout = lambda: None #not using explicit logout now.

    def requestAvatar(self, avatarId, mind, *interfaces):
        """Return resource object given a user's avatarId (username).

        Given a specific level of permissions, the user in question
        gets a different 'view' or 'capabilities' of all the 
        application's total 'capabilities', (possible operations 
        the main administrator could perform).
        """
        if avatarId is checkers.ANONYMOUS:
            r = resources.AnonymousUser(None, self.dbConnection)
            return (iweb.IResource, r, self.logout) #return immediatly if this is not a logged in user.

        if len(mind.kill_kernels) > 0:
            return self._kill_all_users_kernels(mind)

        #kernelConnection = None
        #if mind.is_kernel_operation(): #is this request an operation that is related to computation?
            #kernelConnection = self._get_or_create_kernel(mind)

        avatar = self._get_avatar_object(avatarId, mind)
        return (iweb.IResource, avatar, self.logout)

    def _get_or_create_kernel(self, nbid, dbresult):
        if len(dbresult[0]) == 1:
            return #not a kernel operation
        if nbid in self.kernels: #does there already exist running kernel for this request?
            kernelConnection = self.kernels[nbid]
        else:
            ktype = str(dbresult[0][1])
            kernelConnection = self.kernels[nbid] = kernel.KernelManager(ktype, nbid)
        return kernelConnection

    def _get_avatar_object(self, avatarId, mind):
        """Check privledges and return appropiate avatar.
        """
        if mind.is_kernel_operation():
            query = "SELECT u.permissions, n.kernel FROM users u, notebooks n WHERE u.avatarId = ? AND n.notebookId = ?"
            vals = (avatarId, mind.nbid)
        else:
            query = "SELECT permissions FROM users WHERE avatarId = ?"
            vals = (avatarId,)
        d = self.dbConnection.runQuery(query, vals)
        d.addCallback(self._check_permissions_return_avatar, avatarId, mind)
        return d

    def _check_permissions_return_avatar(self, dbresult, avatarId, mind): #callback for db call.
        kernelConnection = self._get_or_create_kernel(mind.nbid, dbresult)
        permissions = str(dbresult[0][0])
        if permissions == "admin":
            return resources.AdminUser(avatarId, mind.cookie, kernelConnection, self.dbConnection)
        else:
            return resources.RegularUser(avatarId, mind.cookie, kernelConnection, self.dbConnection)

    def _kill_all_users_kernels(self, mind):
        for id in mind.kill_kernels:
            try:
                kernelConnection = self.kernels.pop(id)
                kernelConnection.kill()
            except KeyError:
                pass #a notebook was started, but no cell was evaluated
        return (iweb.IResource, None, self.logout)



