from twisted.cred import checkers, credentials, error
from zope.interface import implements

from knoboo.authority import hashpass

class HashedPasswordDataBaseChecker(object):
    """Compare given plain text passwords with
       hashed ones store in the database.

       Once a user (with username == avatarId) has authenticated,
       we keep the avatarId in an in-memory cache for efficiency.
    """
    implements(checkers.ICredentialsChecker)
    credentialInterfaces = (credentials.IUsernamePassword,)

    def __init__(self, dbConnection):
        self.dbConnection = dbConnection
        self.authcache = {}

    def _passwordCheck(self, result, password):
        if result:
            avatarId, hashedpass = str(result[0][0]), str(result[0][1])
            if hashpass.check_password_against_hash(password, hashedpass):
                self.authcache[(avatarId, password)] = avatarId
                return avatarId #defer.succeed(avatarId)
            else:
                raise error.LoginFailed #user exists, but wrong password.
        else:
            raise error.Unauthorized #no such user exists.

    def requestAvatarId(self, credentials):
        username, password = credentials.username, credentials.password
        try:
            return self.authcache[(username, password)] #XXX Need to remove on logout?
        except KeyError:
            query = "SELECT avatarId, hashedpass FROM users WHERE avatarId = ?"
            d = self.dbConnection.runQuery(query, (username,))
            d.addCallback(self._passwordCheck, password)
            #d.addErrback(self._failed)
            return d


