# Take plain text passwords and return 'salted hashes'.
# The code in the file was inspired by Djangos 'auth' implementation.

import random
import time

def salted_hashed_password(plaintext_password):
    """Return a string consisting of:
        'the python hash module used' + "$" + 'the salt' + "$" + 'the hashed plaintext password'

       We do this to store this data in the database,
       not plain text password.  There exist a great 
       deal of literature all over the internet supporting
       this methodology.
    """
    randstr = "%s_%s" % (str(random.random()),  str(time.time()))
    try:
        import hashlib #Python 2.5
    except ImportError:
        import sha
        module = "sha"
        salt = sha.new(randstr).hexdigest()
        hashedpass = sha.new(salt + plaintext_password).hexdigest()
    else:
        module = "hashlib"
        salt = hashlib.sha1(randstr).hexdigest()
        hashedpass = hashlib.sha1(salt + plaintext_password).hexdigest()
    return module + "$" + salt + "$" + hashedpass

def check_password_against_hash(plaintext_password, password_hash_string):
    """Return a boolean depending on the equality of
    the give plain text password with the hashed password
    """
    if password_hash_string.count("$") != 2:
        raise ValueError("The given password hash string is in an incorrect format.")

    module, salt, hashedpass = password_hash_string.split('$')
    if module == "sha":
        import sha
        attempt = sha.new(salt + plaintext_password).hexdigest() 
    else:
        import hashlib
        attempt = hashlib.sha1(salt + plaintext_password).hexdigest() 
    return attempt == hashedpass

