import os
import getpass
from string import Template
import ConfigParser

import knoboo
k_path = knoboo.__path__[0]

from knoboo.authority import hashpass

class Config(ConfigParser.ConfigParser):
    def __init__(self, twistd=None):
        ConfigParser.ConfigParser.__init__(self)
        home = os.getenv('HOME')
        self._dotkb_path = os.path.join(home, '.knoboo')
        self._conf_name = 'knoboo.conf'
        self._conf_path = os.path.join(self._dotkb_path, self._conf_name)
        self._db_driver = 'sqlite3'
        self._db_main = 'knoboo.db'
        self._db_search = 'search.db'
        self._db_main_path = os.path.join(self._dotkb_path, self._db_main)
        self._db_search_path = os.path.join(self._dotkb_path, self._db_search)
        self._kernel_env_path = os.path.join(self._dotkb_path, 'kernel_env')
        self.config_dict = {'twistd':twistd}

    def read_config(self):
        self.check()
        self.read(self._conf_path)
        self.config_dict = dict(self.items('DEFAULT'))
        return self.config_dict

    def check(self):
        self.check_for_dot_knoboo()
        self.check_for_db()
        self.check_for_conf()

    def check_for_dot_knoboo(self):
        """See if the users home dir contains .knoboo/
        """
        if not os.path.exists(self._dotkb_path):
            self.make_dot_knoboo()

    def make_dot_knoboo(self):
        """make $HOME/.knoboo"""
        os.mkdir(self._dotkb_path)
        os.mkdir(self._kernel_env_path)

    def check_for_conf(self):
        if not os.path.exists(self._conf_path):
            self.make_default_conf()
        
    def make_default_conf(self):
        conffile = open(self._conf_path, 'w')
        dotkb_path = os.path.join(self._kernel_env_path, '')
        d = {'dotkb_path':dotkb_path, 
            'db_main_path':self._db_main_path, 
            'db_search_path':self._db_search_path, 
            'db_driver':self._db_driver}
        def_conf = Template(DEFAULT_CONF).substitute(d)
        conffile.writelines(def_conf)
        conffile.close()

    def make_default_db(self):
        try:
            import sqlite3 as sqlite
        except ImportError:
            from pysqlite2 import dbapi2 as sqlite
            self._db_driver = 'pysqlite2.dbapi2'
        #print 'creating database'
        con = sqlite.connect(self._db_main_path)
        cur = con.cursor()
        #f = open('knoboo/db_setup.sql')
        f = open(os.path.join(k_path, 'db_setup.sql'))
        fs = f.read()
        cur.executescript(fs)
        con.commit()
        f.close()
        self.add_admin_user_to_db(cur, con)
        
    def check_for_db(self):
        if not os.path.exists(self._db_main_path):
            self.make_default_db()

    def add_admin_user_to_db(self, db_cur, db_con):
        print "Enter admin USERNAME. (Hit 'Return' for default 'admin'): "
        #username = raw_input("Enter admin USERNAME. (Hit 'Return' for default 'admin'): ")
        username = raw_input()
        print "Enter admin PASSWORD. (Hit 'Return' for default 'secret'): "
        #password = getpass.getpass("Enter admin PASSWORD. (Hit 'Return' for default 'secret'): ")
        password = getpass.getpass()
        username, password = username.strip(), password.strip()
        if username is None or username == "":
            username = "admin"
        if password is None or password == "":
            password = "secret"
        hashedpass = hashpass.salted_hashed_password(password)
        db_cur.execute("INSERT INTO users VALUES(?, ?, ?, 'admin')", (username, username, hashedpass))
        db_con.commit()
        db_cur.close()

        
DEFAULT_CONF = """
[DEFAULT]
#port that Knoboo server will listen on
port = 8000
#use ssl
secure = 0 
#serve Knoboo behind a proxy like Apache or Nginx
proxy = 0
# for big server, specify static path, blank is default
static_path = 
#path to twistd script
twistd = twistd
#path to kernel module/server
kernel_env = $dotkb_path
#path to main database
db_main_path = $db_main_path
#path to search database
db_search_path = $db_search_path
#path to database diver
db_driver = $db_driver
#development mode: for javascript
devel_mode = 0

"""

            


