CREATE TABLE users (
    avatarId TEXT NOT NULL UNIQUE,
    name text NOT NULL,
    hashedpass text NOT NULL,
    permissions text NOT NULL);

CREATE TABLE settings (
    avatarId TEXT NOT NULL UNIQUE,
    cell_input_border TEXT DEFAULT 'None',
    cell_output_border TEXT DEFAULT 'None');

CREATE TABLE notebooks (
    notebookId TEXT NOT NULL UNIQUE,
    avatarId TEXT NOT NULL,
    creationTime timestamp,
    title TEXT,
    kernel TEXT,
    orderlist TEXT,
    cellsrepr TEXT,
    lastModified timestamp,
    totalCells INT DEFAULT 0, /* total number of input cells*/
    locationId TEXT); /*the bookshelf locationId*/

CREATE TABLE bookshelf_locations (
    locationId TEXT NOT NULL UNIQUE,
    avatarId TEXT NOT NULL,
    creationTime timestamp NOT NULL,/*make not not null*/
    title TEXT);

CREATE TABLE cells (
    cellId TEXT NOT NULL UNIQUE,
    creationTime timestamp NOT NULL ,
    avatarId TEXT NOT NULL,
    notebookId TEXT NOT NULL,
    cellContent TEXT DEFAULT '', 
    cellStyle TEXT DEFAULT '',
    cellProps TEXT DEFAULT '');

CREATE TRIGGER new_notebook AFTER INSERT ON notebooks
    BEGIN
        UPDATE notebooks SET lastModified = DATETIME('now'), 
            creationTime = DATETIME('now'),
            orderlist = 'orderlist'
        WHERE notebookID = new.notebookId;
    END;

CREATE TRIGGER new_settings AFTER INSERT ON users 
    BEGIN
        INSERT INTO settings (avatarId, cell_input_border, cell_output_border) 
        VALUES (new.avatarId, 'None', 'None');
    END;

CREATE TRIGGER cell_insert AFTER INSERT ON cells
    BEGIN
        UPDATE notebooks SET 
            lastModified = DATETIME('now'), 
            totalCells = (SELECT COUNT(cellContent) FROM cells 
            WHERE notebookId = new.notebookId AND cellStyle = 'input')   
        WHERE notebookId = new.notebookId; 
    END;

CREATE TRIGGER cell_delete AFTER DELETE ON cells
    BEGIN
        UPDATE notebooks SET 
            lastModified = DATETIME('now'), 
            totalCells = (SELECT COUNT(cellContent) FROM cells 
            WHERE notebookId = old.notebookId AND cellStyle = 'input')   
        WHERE notebookId = old.notebookId; 
    END;

