# Indexer for search of a users notebooks
import os
import datetime
import time
import string
try:
    import sqlite3 as sqlite
except ImportError:
    from pysqlite2 import dbapi2 as sqlite


CREATE_TABLE0 = """\
CREATE TABLE documents (
    doc_id TEXT NOT NULL UNIQUE,
    doc_len INT DEFAULT 0);
"""
CREATE_TABLE1 = """\
CREATE TABLE occurance (
    word TEXT,
    doc_id TEXT,
    occur INT DEFAULT 0,
    avatarId TEXT);
"""

NOTLETTERS = string.punctuation+''.join([str(i) for i in range(10)])

def get_db_cur_con(dbname):
    if dbname == "knoboo.db":
        db_path = os.path.join(os.getenv('HOME'), '.knoboo/knoboo.db')
        con = sqlite.connect(db_path)
        cur = con.cursor()
    elif dbname == "search.db":
        db_path = os.path.join(os.getenv('HOME'), '.knoboo/search.db')
        if not os.path.exists(db_path):
            con = sqlite.connect(db_path)
            cur = con.cursor()
            cur.execute(CREATE_TABLE0)
            cur.execute(CREATE_TABLE1)
            con.commit()
        else:
            con = sqlite.connect(db_path)
            cur = con.cursor()
    return (cur, con)

def get_all_users(dbcursor):
    query = "SELECT avatarId FROM users"
    dbcursor.execute(query)
    return dbcursor.fetchall()

def get_all_users_notebook_ids(dbcursor, user):
    query = "SELECT notebookId, lastModified FROM notebooks WHERE avatarId = ?"
    dbcursor.execute(query, (user,))
    return dbcursor.fetchall()

def get_all_cells_with_nbid(dbcursor, nbid):
    query = "SELECT cellContent FROM cells WHERE notebookId = ?"
    dbcursor.execute(query, (nbid,))
    return dbcursor.fetchall()

def all_words_in_notebook(cells):
    words = []
    s = ' '.join([str(data) for data in cells])
    for char in NOTLETTERS:
        s = s.replace(char, ' ')
    for w in s.split(' '):
        if len(w) > 0:
            words.append(w)
    return words

def count_word_occurances(words):
    count = {}
    unique_words = list(set(words))
    for word in unique_words:
        count[word] = words.count(word)
    return count

def insert_nb_word_occur(dbcursor, count, nbid, avatarId):
    values = [(word, nbid, occur, avatarId) for word, occur in count.items()]
    sql = "REPLACE INTO occurance VALUES(?, ?, ?, ?)"
    dbcursor.executemany(sql, values)


CUR_KNOBOO, CON_KNOBOO = get_db_cur_con("knoboo.db")
CUR_SEARCH, CON_SEARCH = get_db_cur_con("search.db")

def latest():
    CUR_KNOBOO.execute("SELECT MAX(lastModified) from notebooks")
    res = CUR_KNOBOO.fetchall()
    mtime = res[0][0]
    if mtime is None:
        return datetime.datetime.now()
    mtime = mtime.split('.')[0]
    return datetime.datetime.strptime(str(mtime), "%Y-%m-%d %H:%M:%S")

def indexer():
    allusers = get_all_users(CUR_KNOBOO)
    for user in allusers:
        user = str(user[0])
        allnbids = get_all_users_notebook_ids(CUR_KNOBOO, user)
        for nbid in allnbids:
            nbid = str(nbid[0])
            cells = get_all_cells_with_nbid(CUR_KNOBOO, nbid)
            words = all_words_in_notebook(cells)
            count = count_word_occurances(words)
            insert_nb_word_occur(CUR_SEARCH, count, nbid, user)
            CUR_SEARCH.execute("REPLACE INTO documents VALUES(?, ?)", (nbid, len(words))) 
            CON_SEARCH.commit()
    #CON_SEARCH.close()
    #CON_KNOBOO.close()

def update(lastest):
    allusers = get_all_users(CUR_KNOBOO)
    for user in allusers:
        user = str(user[0])
        allnbids = get_all_users_notebook_ids(CUR_KNOBOO, user)
        for tup in allnbids:
            nbid = str(tup[0])
            mtime = str(tup[1]).split('.')[0]
            nblastmod = datetime.datetime.strptime(mtime, "%Y-%m-%d %H:%M:%S")

            if nblastmod > lastest:
                cells = get_all_cells_with_nbid(CUR_KNOBOO, nbid)
                words = all_words_in_notebook(cells)
                count = count_word_occurances(words)
                insert_nb_word_occur(CUR_SEARCH, count, nbid, user)
                CUR_SEARCH.execute("REPLACE INTO documents VALUES(?, ?)", (nbid, len(words))) 
                CON_SEARCH.commit()
    #CON_SEARCH.close()
    #CON_KNOBOO.close()



