"""Generate a Python Interpreter Kernel RPC Service

This server provides an XML-RPC interface to a Python Interpreter.

Jan 2008:
Newest version uses python SimpleXMLRPCServer

Old version:
The server uses the knoboo.external.twisted.web2 framework.

The Kernel subclasses knoboo.external.twisted.web2's xmlrpc.XMLRPC [protocol something]
The kernel provides procedures for executing basic general things in python

"""

def generate_kernel_server_file(rundir, kernel_file_name, port):
    """Generate a server with the name: kernel_file_name listening on
    port:port. Write the server to the specified rundirectory.
    The server process can then be started by the process that called this
    function"""
    from string import Template
    import os
    # fpath = os.path.join(rundir, kernel_file_name)
    kernel_dir_path = str(rundir)+str(kernel_file_name)
    print kernel_dir_path
    kernel_file_path = os.path.join(kernel_dir_path, str(kernel_file_name))
    print kernel_file_path
    # make a dir for this nbid if one does not exist
    if not os.path.exists(kernel_dir_path):
        os.mkdir(kernel_dir_path)

    fout = open(kernel_file_path, 'w')
    filebody = Template(KERNEL_SERVER).substitute({"port":port})
    fout.write(filebody)
    fout.close()

KERNEL_SERVER = """
import time
import sys
import traceback
import logging as log
from SimpleXMLRPCServer import SimpleXMLRPCServer

#sys.path.append('.')


# local import

from knoboo.kernel.python_kernel.python_interpreter import PythonInterpreter

try:
    from knoboo.external.mmaplotlib import plot
    from knoboo.external.mmaplotlib import knoboo_plot
    USERNAMESPACE={"plot":plot, "show":knoboo_plot.show}
except ImportError:
    USERNAMESPACE={}

class KernelConnection(SimpleXMLRPCServer):

    def __init__(self, addr, userNamespace=USERNAMESPACE):
        SimpleXMLRPCServer.__init__(self, addr)
        self.userNamespace = userNamespace
        self.p = PythonInterpreter(self.userNamespace)

    def _dispatch(self, method, params):
        try:
            func = getattr(self, 'xmlrpc_' + method)
        except AttributeError:
            raise Exception('method %s is not supported' % method)
        else:
            return func(*params)

    def xmlrpc_evalinput(self, source):
        \"""Evaluate code in the python interpreter.
        return a dict containing:
            - stdout 
            - stderr
            - original input (source)
            - number of commands source contained\"""
        result = self.p.execute(source)
        # return result['out']+result['err']
        return result

    def xmlrpc_cancelinterrupt(self):
        result = self.p.cancel_interrupt()
        return result
    
    def xmlrpc_completename(self, source):
        \"""Search for possible completion matches of source in the
        usernamespace.
        return a list of matches\"""
        result = self.p.complete_name(source)
        return result

    def xmlrpc_completeattr(self, source):
        \"""Get the attributes/methods of an object (source).
        return them in a list\"""
        result = self.p.complete_attr(source)
        return result


if __name__ == "__main__":
    #import signal
    #oldusr1=signal.signal(signal.USR1, usr1_interrupt)
    print "Python XML-RPC Kernel Serving on port $port"
    server = KernelConnection(('localhost', $port))
    server.serve_forever()
"""

# OLD TWISTED VERSION
# KERNEL_FILE = "kernel_python.tac"
KERNEL_SERVER_twisted = """
from knoboo.external.twisted.web2 import http_headers, responsecode
from knoboo.external.twisted.web2 import resource
from knoboo.external.twisted.web2 import channel
from knoboo.external.twisted.web2 import stream
from knoboo.external.twisted.web2 import server
from knoboo.external.twisted.web2 import http
from knoboo.external.twisted.web2 import static
from knoboo.external.twisted.web2 import xmlrpc
from twisted.application import service, strports
from twisted.python import log

import time
import sys
import traceback

sys.path.append('.')

# local import

from knoboo.kernel.python_kernel.python_interpreter import PythonInterpreter

try:
    from knoboo.external.mmaplotlib import plot
    from knoboo.external.mmaplotlib import knoboo_plot
    USERNAMESPACE={"plot":plot, "show":knoboo_plot.show}
except ImportError:
    USERNAMESPACE={}

class KernelConnection(xmlrpc.XMLRPC):

    addSlash = True
    def __init__(self, userNamespace=USERNAMESPACE):
        self.userNamespace = userNamespace
        self.p = PythonInterpreter(self.userNamespace)

    def xmlrpc_evalinput(self, request, source):
        \"""Evaluate code in the python interpreter.
        return a dict containing:
            - stdout 
            - stderr
            - original input (source)
            - number of commands source contained\"""
        result = self.p.execute(source)
        log.msg(result)
        # return result['out']+result['err']
        return result
    
    def xmlrpc_completename(self, request, source):
        \"""Search for possible completion matches of source in the
        usernamespace.
        return a list of matches\"""
        result = self.p.complete_name(source)
        log.msg(result)
        return result

    def xmlrpc_completeattr(self, request, source):
        \"""Get the attributes/methods of an object (source).
        return them in a list\"""
        result = self.p.complete_attr(source)
        log.msg(result)
        return result

site = server.Site(KernelConnection())
application = service.Application("Example XML-RPC Server")
s = strports.service('tcp:$port:interface=127.0.0.1', channel.HTTPFactory(site))
s.setServiceParent(application)
"""
