import math

### Ignore Words ###
PREPOSITIONS = ['on', 'after', 'under', 'for']
CONJUNCTIONS = ['and', 'if', 'but', 'so']


### Weights ###

#Collection Frequency Weight
def cfw(n, N):
    """Terms that only occur in a few documents have
       more weight than those that occur in many.

       n = # documents term t(i) occurs in. (int)
       N = # documents in collection. (int)
    """
    return math.log(N) - math.log(n)

#Term Frequency
def tf_T(ti, dj):
    """How many times does the ith term (ti) occur
       in the jth document (dj).  The more times
       a term appears in a doc, the greater importance.

       ti = the ith term (string)
       dj = list of documents relevant words (list of strings)
    """
    return dj.count(ti)

#Normalized Document Length
def ndl_T(djl, adl):
    """The ratio of the jth document length (dlj) by
       the average document length of all documents (adl)

       djl = jth document length (int)
       adl = average document length (int)
    """
    return djl/adl


#Combined Weight
def cw(i, tf, ndl, K1=2.0, b=0.75):
    cfw(i) * tf * (K1 + 1) / (K1 * ((1 - b) + (b * ndl)) + tf)




# Notes
"""
 * Each word can be contained in many documents (1 word -> many docs)
 
 === db schema ===

 -table words

 -table documents /* updated on parse */
   doc_id (str unique)
   length (int)

 -occurance 
   word (str)
   doc_id (str)
   occur (int) /* how many times does 'word' appear in 'doc_id' */
   (doc_id && word  unique)

 === db queries ===

 /*  N = (# documents in collection), smd = (sum all document lengths), adl = (N/smd - Average Document Length)  */
 select COUNT(doc_id), SUM(length) from documents

 /*   ndl =   dlj/adl   - Normalized Document Length  */
 select doc_id, occur from occurance WHERE word = '?'

"""

