##################################################################### 
# Copyright (C) 2007 Alex Clemesha <clemesha@gmail.com>
# 
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
##################################################################### 
from knoboo.external.twisted.web2 import http_headers
from knoboo.external.twisted.web2 import responsecode
from knoboo.external.twisted.web2 import resource
from knoboo.external.twisted.web2 import http
#standard library
import os
import time
import datetime
import random
import md5

#json encoding
#import simplejson as json
from knoboo.external import minjson as json

#template for the html pages
from knoboo.templates import templates

#sql static queries
import knoboo.sql as sql

#config
from knoboo import config
kbconfig = config.Config()
cfg = kbconfig.read_config()

CTYPE_UTF8 = {'content-type': http_headers.MimeType('text', 'html', {'charset':'utf-8'})}
CTYPE_JSON = {'content-type': http_headers.MimeType('application', 'json', {'charset':'utf-8'})}


class SignUp(resource.Resource):
    def render(self, request):
        html = templates.signup()         
        return http.Response(responsecode.OK, CTYPE_UTF8, html)

class CreateAccount(resource.PostableResource):
    """Let anyone create an account.

    For security, this resource must be thought more about.

    One thought is that we could let anyone create an account,
    but then allow/restrict in the actually login checking logic,
    found in avatars.py.

    As of now, this is the only place where an anonymous user
    can access the database, i.e. when some users creates an account,
    his information is written to the database.

    For a more secure deployment, this probably should at least
    have very careful checking on the incoming input.
    """
    _un_length = 3 #username min length
    _pw_length = 5 #password min length
    
    def __init__(self, dbConnection):
        self.dbConnection = dbConnection

    def render(self, request):
        username = request.args.get('email', [''])[0]
        password1 = request.args.get('password', [''])[0]
        password2 = request.args.get('password2', [''])[0]

        errors = self._check_info(username, password1, password2)
        if errors is not None:
            #html = templates.signup(errors)         
            msg = str({"errors":errors})
            return http.Response(responsecode.OK, CTYPE_JSON, msg)

    def _check_info(self, username, password1, password2):
            if len(username) < self._un_length:
                return "Username is missing or is too short."
            if len(password1) < self._pw_length or len(password2) < self._pw_length:
                return "Passwords are missing or they are too short."
            if password1 != password2:
                return "Passwords are not equal."
            else:
                print(username, password1, password2)
                return "OK"

        #d = self.dbConnection.runQuery(sql.CREATE_ACCOUNT, values)
        #d.addCallback(self._success)
        #d.addErrback(self._failed)
        #return d

    def _success(self, result):
        redirect = "/newaccount?user=" + self._user + "&status=ok"
        response = http.RedirectResponse(redirect)
        return response


