##################################################################### 
# Copyright (C) 2007 Alex Clemesha <clemesha@gmail.com>
# 
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
##################################################################### 
#standard library
import os
import time
import datetime
import random
import md5

#twisted web2
from knoboo.external.twisted.web2 import http_headers, responsecode
from knoboo.external.twisted.web2 import resource
from knoboo.external.twisted.web2 import http
from knoboo.external.twisted.web2 import static 

#for remote file downloading
from twisted.web.client import getPage

#json encoding
#import simplejson as json
from knoboo.external import minjson as json
#template for the html pages
from knoboo.templates import templates
#knoboo resources
from knoboo.resources import search
#sql static queries
from knoboo import sql

#config
from knoboo import config
kbconfig = config.Config()
cfg = kbconfig.read_config()

CTYPE_UTF8 = {'content-type': http_headers.MimeType('text', 'html', {'charset':'utf-8'})}
CTYPE_JSON = {'content-type': http_headers.MimeType('application', 'json', {'charset':'utf-8'})}

# ['nbid', 'title', 'kernel', 'number of cells', 'last modified', 'view']
# 'view' is a notebooks current location {'all':0, 'trash':1, 'archive':2, 'folder':3}, //folder needs to be generalized

class BookShelf(resource.Resource):
    """The listing of a user's notebooks.
    """
    addSlash = True
    def __init__(self, avatarId, dbConnection):
        self.avatarId= avatarId
        self.child_static = static.File(os.path.abspath(".")+"/knoboo/static/bookshelf")
        #self.child_uploadnb = UploadNotebook()
        self.child_new = CreateNewNotebook(avatarId, dbConnection)
        self.child_newfolder = CreateNewFolder(avatarId, dbConnection)
        self.child_renamefolder = RenameUserFolder(avatarId, dbConnection)
        self.child_move = SetNotebookLocation(avatarId, dbConnection)
        self.child_deletenb = DeleteNotebooks(avatarId, dbConnection)
        self.child_load = LoadUserBookShelfData(avatarId, dbConnection)    
        self.child_userfolders = LoadUserFolders(avatarId, dbConnection)    
        self.child_attach = Attach(avatarId, dbConnection)    
        self.child_attachremote = AttachRemote(avatarId, dbConnection)    
        self.child_search = search.Search(avatarId)    

    def render(self, req):
        html = templates.bookshelf(self.avatarId)
        return http.Response(responsecode.OK, CTYPE_UTF8, html)


class CreateNewNotebook(resource.PostableResource):

    def __init__(self, avatarId, dbConnection):
        self.avatarId= avatarId
        self.dbConnection = dbConnection

    def render(self, request):
        self.kernel = request.args.get("kernel", ["python"])[0]
        #timenow = int(request.args.get("timenow", [""])[0])
        #created = datetime.datetime.fromtimestamp(int(timenow / 1000))
        #created = datetime.datetime.now()
        self.nbid = self._create_id()
        #values = (self.nbid, self.avatarId, created, "Untitled", self.kernel, "orderlist", "cellsrepr", created, 0, 'all')
        values = (self.nbid, self.avatarId, "Untitled", self.kernel)
        d = self.dbConnection.runQuery(sql.CREATE_NOTEBOOK, values)
        d.addCallback(self._success)
        #d.addErrback(self._failed)
        return d

    def _success(self, result):
        path = "/notebook?id=%s&system=%s" % (self.nbid, self.kernel)
        return http.RedirectResponse(path)

    def _create_id(self):
        data = "%s_%s" % (str(random.random()) , str(time.time()))
        return md5.new(data).hexdigest()

class CreateNewFolder(resource.PostableResource):

    def __init__(self, avatarId, dbConnection):
        self.avatarId = avatarId
        self.dbConnection = dbConnection

    def render(self, request):
        self.name = request.args.get('name', ['New Folder'])[0]
        createTime = datetime.datetime.now()
        self.id = self._create_id()
        values = (self.id, self.avatarId, createTime, self.name)
        d = self.dbConnection.runOperation(sql.CREATE_FOLDER, values)
        d.addCallback(self._success)
        return d

    def _success(self, result):
        obj = json.write([[self.id,self.name]], outputEncoding='utf-8')
        return http.Response(responsecode.OK, CTYPE_JSON, obj)


    def _create_id(self):
        data = "%s_%s" % (str(random.random()) , str(time.time()))
        return md5.new(data).hexdigest()

class DeleteNotebooks(resource.PostableResource):
    """Delete notebooks from the database.

    Take a list of notebook ids, and deleted all 
    data associated with the id.
    """

    def __init__(self, avatarId, dbConnection):
        self.avatarId = avatarId
        self.dbConnection = dbConnection

    def _deletedSuccess(self, result):
        jsondata = "{'response':'ok'}"
        return http.Response(responsecode.OK, CTYPE_JSON, jsondata)

    def _deletedFailed(self, result):
        jsondata = "{'response':'failed'}"
        return http.Response(responsecode.OK, CTYPE_JSON, jsondata)

    def render(self, req): 
        """What initially handles the request.
        Get all the post args and go into the chain of defereds.
        """
        nbids = req.args['nbid']
        operation = "DELETE FROM notebooks WHERE notebookId = ?"
        #XXX some connections aren't closing???
        dl = [self.dbConnection.runOperation(operation, (id,)) for id in nbids]
        d = defer.DeferredList(dl)
        d.addCallback(self._deletedSuccess)
        return d

class SetNotebookLocation(resource.PostableResource):
    """Set/change the folder/location of a notebook.
    """

    def __init__(self, avatarId, dbConnection):
        self.avatarId= avatarId
        self.dbConnection = dbConnection

    def render(self, request):
        """the args:
        ?dest=N&nbid=1&nbid=2[...] -> {'dest':['N'], 'nbid': ['1', '2']}
        """
        query = "UPDATE notebooks SET locationId = ? WHERE notebookId = ?"

        dest = request.args['dest'][0]
        nbids = request.args['nbid']
        data = [(str(dest), str(id)) for id in nbids]

        #d = self.dbConnection.runOperation(query, vals)
        #the below function essentially does what
        #an 'executemany' would do in SQL.
        def daisy_chain(res):
            try:
               vals = data.pop()
            except KeyError:
                return self._success
            dd = self.dbConnection.runOperation(query, vals)
            if data:
                dd.addCallback(daisy_chain)
                return dd
            else:
                dd.addCallback(self._success)
                return dd
        #d.addCallback(daisy_chain)
        return daisy_chain('')

    def _success(self, result):
        return http.Response(responsecode.OK, CTYPE_JSON, "{result:'ok'}")

class AttachRemote(resource.Resource):
    """Download a remote data-file for use in user's notebooks.

    """ 

    def __init__(self, avatarId, dbConnection):
        self.avatarId= avatarId
        self.dbConnection = dbConnection

    def render(self, request):
        url = request.args.get("url", [""])[0]
        d = getPage(url)
        d.addCallback(self._getSuccess, url)
        d.addErrback(self._getFailed)
        return d

    def _getFailed(self, res):
        return http.Response(responsecode.OK, CTYPE_JSON, "{result:'failed'}")

    def _getSuccess(self, res, url):
        msg = "{result:'ok'}"
        try:
            fname =  url.split("/")[-1]
            f = open(fname, "w"); f.write(res); f.close()
        except:
            msg = "{result:'failed'}"
        return http.Response(responsecode.OK, CTYPE_JSON, msg)


class Attach(resource.PostableResource):
    iframehtml = """\
    <html>
    <body>
    <form id="uploadform" enctype="multipart/form-data" method="post">
        <p>Attach data from your computer:</p>
        <input type='file' name='localfile' width='60' />
        <input type='submit' id='upload_btn' value='Attach Local File'></input>
     </form>
    </body>
    </html>
    """

    def __init__(self, avatarId, dbConnection):
        self.avatarId= avatarId
        self.dbConnection = dbConnection

    def render(self, request):
        file = request.files.get('localfile', '')
        filename, filedata = None, None
        if file:
            filename = file[0][0]
            filedata = file[0][2]
            ext = filename[-3:].strip().lower()
            return self._get_kernels(filename, filedata)
        return http.Response(responsecode.OK, CTYPE_UTF8, self.iframehtml)

    def _get_kernels(self, filename, filedata):
        query = "SELECT notebookId FROM notebooks WHERE avatarId = ?"
        d = self.dbConnection.runQuery(query, (self.avatarId,))
        d.addCallback(self._put_attached_data, filename, filedata)
        return d

    def _put_attached_data(self, res, filename, filedata):
        filedata = filedata.read()
        kernel_env = cfg['kernel_env']
        for nbid in res:
            nbid = str(nbid[0])
            abspath = kernel_env + nbid + '/' + filename
            f = open(abspath, 'w')
            f.write(filedata)
            f.close()
        return http.Response(responsecode.OK, CTYPE_UTF8, self.iframehtml)


class LoadUserBookShelfData(resource.Resource):
    """Return a subset of a users notebooks.

       For a given set of query params, return a
       a JSON object containing data corresponding
       to all notebooks matching the query.

       Possible query params:

           'order' => (some_notebook_attr) | default -> 'date'
           'sort' => ('asc', 'desc') | default -> 'desc'

           'limit' => (some_integer > 0) | default -> 20
           'offset' => (some_integer > 0 ) | default -> -1

           'newest' => (some_timestamp) | no default

           'location' => (some_int OR some_id)
               0 -> 'all'
               1 -> 'trash'
               2 -> 'archive'
    """
    MAX_NOTEBOOKS = 20

    order_map = {
        'date':'lastModified',
        'cells':'totalCells',
        'title':'title', 
        'kernel':'kernel' 
    }

    query_section = """SELECT notebookId, title, kernel, totalCells, lastModified, locationId
                       FROM notebooks WHERE avatarId = ? AND locationId = ?  ORDER BY %s LIMIT ? OFFSET ?"""

    query_section_all = """SELECT notebookId, title, kernel, totalCells, lastModified, locationId
                       FROM notebooks WHERE avatarId = ? ORDER BY %s LIMIT ? OFFSET ?"""

    query_newer = """SELECT notebookId, title, kernel, totalCells, lastModified, locationId
                     FROM notebooks WHERE avatarId = ? AND creationTime > ? ORDER BY %s LIMIT ? OFFSET ?"""

    def __init__(self, avatarId, dbConnection):
        self.avatarId = avatarId
        self.dbConnection = dbConnection

    def render(self, request):
        query, args = self._get_query_args(request)
        d = self.dbConnection.runQuery(query, args)
        d.addCallback(self._success)
        #d.addErrback(self._failed)
        return d

    def _get_query_args(self, request):
        """
        Get sort attributes from query args dict.
        """
        location = request.args.get('location')[0]
        sort = request.args.get('sort',['desc'])[0]
        order = request.args.get('order', ['date'])[0]
        order = self.order_map.get(order) + ' ' + sort
        limit = request.args.get('limit', [self.MAX_NOTEBOOKS])[0]
        offset = request.args.get('offset', [0])[0]

        #we are going to have either a 'location'
        #query arg, or we will have a 'newest' timestamp,
        #and depending on which one we have will determine
        #which type of query we will be doing:
        if location == 'all':
            query_args = (self.avatarId, limit, offset)
            query = self.query_section_all % order
            return [query, query_args] 

        try:
            location = request.args["location"][0]
            #query_args = (self.avatarId, location, order, limit, offset)
            query_args = (self.avatarId, location, limit, offset)
            #query = self.query_section % sort.upper()
            query = self.query_section % order
            return [query, query_args] 
        except KeyError:
            newest = request.args.get("newest", ["1970-01-01 00:00:01"])[0]
            #query_args = (self.avatarId, newest, order, limit, offset)
            query_args = (self.avatarId, newest, limit, offset)
            #query = self.query_newer % sort
            query = self.query_newer % order
            return [query, query_args] 

    def _success(self, result):
        obj = json.write(result, outputEncoding='utf-8')
        return http.Response(responsecode.OK, CTYPE_JSON, obj)


class LoadUserFolders(resource.Resource):

    def __init__(self, avatarId, dbConnection):
        self.avatarId = avatarId
        self.dbConnection = dbConnection

    def render(self, request):
        values = (self.avatarId,)
        d = self.dbConnection.runQuery(sql.USER_LOCATIONS, values)
        d.addCallback(self._success)
        return d

    def _success(self, result):
        obj = json.write(result, outputEncoding='utf-8')
        return http.Response(responsecode.OK, CTYPE_JSON, obj)


class RenameUserFolder(resource.PostableResource):

    def __init__(self, avatarId, dbConnection):
        self.avatarId= avatarId
        self.dbConnection = dbConnection

    def render(self, request):
        self.id = request.args.get('id')[0]
        self.name = request.args.get('name')[0]
        values = (self.name, self.id)
        d = self.dbConnection.runOperation(sql.RENAME_FOLDER, values)
        d.addCallback(self._success)
        return d

    def _success(self, result):
        obj = json.write([[self.id,self.name]], outputEncoding='utf-8')
        return http.Response(responsecode.OK, CTYPE_JSON, obj)





