##################################################################### 
# Copyright (C) 2007 Alex Clemesha <clemesha@gmail.com>
# 
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
##################################################################### 
from knoboo.external.twisted.web2 import http_headers, responsecode
from knoboo.external.twisted.web2 import resource
from knoboo.external.twisted.web2 import http
from knoboo.external.twisted.web2 import channel
from knoboo.external.twisted.web2 import server
from knoboo.external.twisted.web2 import static 
from knoboo.external.twisted.web2 import iweb 
from knoboo.external.twisted.web2.error import ERROR_MESSAGES
#twisted defer
from twisted.internet import defer
#standard library
import os
import time
import datetime
import random
import md5
#interface
from zope.interface import Interface, implements

#twisted database acccess
from twisted.enterprise import adbapi
#json encoding
#import simplejson as json
from knoboo.external import minjson as json

#template for the html pages
from knoboo.templates import templates

#sql static queries
import knoboo.sql as sql

#config
from knoboo import config
kbconfig = config.Config()
cfg = kbconfig.read_config()

CTYPE_UTF8 = {'content-type': http_headers.MimeType('text', 'html', {'charset':'utf-8'})}
CTYPE_JSON = {'content-type': http_headers.MimeType('application', 'json', {'charset':'utf-8'})}


class Notebook(resource.Resource):
    def __init__(self, avatarId, dbConnection):
        self.avatarId = avatarId
        self.dbConnection = dbConnection

    def render(self, request):
        """Get the title of the notebook with the notebook id.
        """
        self.nbid = request.args['id'][0]
        self.system = request.args['system'][0]
        d = self.dbConnection.runQuery(sql.NOTEBOOK, (self.nbid, ))
        d.addCallback(self._success)
        #d.addErrback(self._failed)
        return d

    def _success(self, result):
        title = ""
        if result:
            title = str(result[0][0])
        html = templates.notebook(self.avatarId, title, self.nbid, self.system)
        response = http.Response(responsecode.OK, CTYPE_UTF8, html)
        return response


class NotebookObject(resource.Resource):
    """Notebook js object."""
    def __init__(self, avatarId, dbConnection):
        self.avatarId = avatarId
        self.dbConnection = dbConnection

    def render(self, request):
        self.nbid = request.args['id'][0]
        d = self.dbConnection.runQuery(sql.NOTEBOOK_OBJECT, (self.nbid, self.avatarId))
        d.addCallback(self._getCellsData)
        #d.addErrback(self._failed)
        return d

    def _getCellsData(self, result):
        self._orderlist, self._cib, self._cob = result[0]
        d = self.dbConnection.runQuery(sql.NOTEBOOK_OBJECT_CELLSDATA , (self.nbid,))
        d.addCallback(self._success)
        #d.addErrback(self._failed)
        return d

    def _success(self, result):
        nbdata, cellsdata = {}, {}
        if result:
            for cellId, content, style, props in result:
                content = content.decode('string_escape')
                props = props.decode('string_escape') # does this need decoding?
                cellsdata[cellId] = {'content':content, 'cellstyle':style, 'props':props}
        nbdata['cells'] = cellsdata
        nbdata['settings'] = {'cell_input_border':self._cib, 'cell_output_border':self._cob}
        nbdata['nbid'] = self.nbid
        nbdata['orderlist'] = self._orderlist
        nbobj = json.write(nbdata, outputEncoding='utf-8')
        response = http.Response(responsecode.OK, CTYPE_JSON, nbobj)
        return response

class NotebookOperations(resource.Resource):
    """Notebook operations.
    """
    addSlash = True

    def __init__(self, avatarId, kernelConnection, dbConnection): 
        self.avatarId = avatarId
        self.kernelConnection = kernelConnection
        self.dbConnection = dbConnection
        self.child_nbobject = NotebookObject(self.avatarId, self.dbConnection)
        self.child_eval = EvalCell(self.avatarId, self.kernelConnection, self.dbConnection)
        self.child_delete = DeleteCell(self.avatarId, self.kernelConnection, self.dbConnection)
        self.child_change = ChangeNotebookMetaData(self.avatarId, self.kernelConnection, self.dbConnection)
        self.child_completer = TabCompleter(self.avatarId, self.kernelConnection, self.dbConnection)
        self.child_save = SaveNotebookMetaData(self.avatarId, self.dbConnection)
        self.child_kernel = KernelControl(self.avatarId, self.kernelConnection, self.dbConnection)

    def render(self, request):
        child = getattr(self, 'child_%s' % (segments[0],), None)
        if child:
            rsrc = iweb.IResource(child, None)
            if rsrc:
                return rsrc
        return None
        

class EvalCell(resource.PostableResource):
    """
    Each notebook has a unique connection to its kernel.
    Send eval requests to the kernel and wait for the result.
    Write the result (output)and the request code (input) to the database.
    """
    def __init__(self, avatarId, kernelConnection, dbConnection):
        self.avatarId = avatarId
        self.kernelConnection = kernelConnection
        self.dbConnection = dbConnection

    def _saveCells(self, result):
        now = datetime.datetime.now()
        inputcontent = self.input.encode('string_escape')
        inputstyle = 'input'
        inputprops = 'props'
        inputvalues = (self.cellid,\
                now,\
                self.avatarId,\
                self.nbid,\
                inputcontent,\
                inputstyle,\
                inputprops)
        di = self.dbConnection.runOperation(sql.SAVE_CELL, inputvalues)
        print(result)
        self.count = str(result['input_count'])
        output = str(result['out'])
        err = str(result['err'])
        self.output = output + err
        self.outputstyle = ''
        if len(self.output) > 0:
            outcellid = self.cellid + 'o'
            if "__image__" in output:
                self.output = output[9:] + err
                outputstyle = "outputimage"
            else:
                outputstyle = "outputtext"
            self.outputstyle = outputstyle
            outputcontent = self.output.encode('string_escape')
            outputprops = 'props'
            outputvalues = (outcellid,\
                    now,\
                    self.avatarId,\
                    self.nbid,\
                    outputcontent,\
                    outputstyle,\
                    outputprops)
            di.addCallback(lambda res: self.dbConnection.runOperation(sql.SAVE_CELL, outputvalues))
        di.addCallback(self.returnResult)
        return di

    def _saveOutput(self, result):
        now = datetime.datetime.now()
        self.count = str(result['input_count'])
        output = str(result['out'])
        err = str(result['err'])
        self.output = output + err
        self.outputstyle = ''
        if len(self.output) > 0:
            outcellid = self.cellid + 'o'
            if "__image__" in output:
                self.output = output[9:] + err
                outputstyle = "outputimage"
            else:
                outputstyle = "outputtext"
            self.outputstyle = outputstyle
            outputcontent = self.output.encode('string_escape')
            outputprops = 'props'
            outputvalues = (outcellid,\
                    now,\
                    self.avatarId,\
                    self.nbid,\
                    outputcontent,\
                    outputstyle,\
                    outputprops)
            d = self.dbConnection.runOperation(sql.SAVE_CELL, outputvalues)
            d.addCallback(lambda res: res)
            return d
        return result

    def _saveInput(self):
        now = datetime.datetime.now()
        inputcontent = self.input.encode('string_escape')
        inputstyle = 'input'
        inputprops = 'props'
        inputvalues = (self.cellid,\
                now,\
                self.avatarId,\
                self.nbid,\
                inputcontent,\
                inputstyle,\
                inputprops)
        d = self.dbConnection.runOperation(sql.SAVE_CELL, inputvalues)
        return d

    def _do_eval(self, result):
        K = self.kernelConnection.eval(self.input)
        K.addCallback(self._saveOutput)
        K.addErrback(self._kernelErr)
        K.addCallback(self.returnResult)
        return K

    def _kernelErr(self, result):
        self.output = 'Kernel Error'
        self.count = '!'
        self.outputstyle = 'outputtext'
        #TODO Make an errortext style 
        # It could be cool to have tracebacks be errors and
        # kernel errors could be some other sort of message/warning
        return True

    def returnResult(self, result):
        """Return result of some computation
        The last function in the chain of callbacks. 
        """
        print('==output==', self.output)
        data = {'content':self.output, 'count':self.count, 'cellstyle':self.outputstyle, 'cellid':self.cellid}
        jsondata = json.write(data, outputEncoding='utf-8')
        response = http.Response(responsecode.OK, CTYPE_JSON, jsondata)
        return response
 
    def render(self, req): 
        """What initially handles the request.
        Get all the post args and go into the chain of defereds.
        """
        self.cellid = req.args['cellid'][0]
        self.nbid = req.args['nbid'][0]
        self.input = req.args['input'][0].strip()
        d = self._saveInput()
        d.addCallback(self._do_eval)
        return d

class DeleteCell(resource.PostableResource):
    """Delete cells from the database.

    Take a list of cellids, and deleted all 
    data associated with the cellid.
    """

    def __init__(self, avatarId, kernelConnection, dbConnection):
        self.avatarId = avatarId
        self.kernelConnection = kernelConnection
        self.dbConnection = dbConnection

    def _cellsDeleted(self, result):
        jsondata = "{'response':'ok'}"
        response = http.Response(responsecode.OK, CTYPE_JSON, jsondata)
        return response

    def _deletedFailed(self, result):
        jsondata = "{'response':'failed'}"
        response = http.Response(responsecode.OK, CTYPE_JSON, jsondata)
        return response

    def render(self, req): 
        """What initially handles the request.
        Get all the post args and go into the chain of defereds.
        """
        nbid = req.args['nbid'][0]
        cellids = json.read(req.args['cellids'][0])
        #XXX some connections aren't closing???
        dl = [self.dbConnection.runOperation(sql.DELETE_CELL, (id, self.avatarId)) for id in cellids]
        d = defer.DeferredList(dl)
        d.addCallback(self._cellsDeleted)
        return d


class ChangeNotebookMetaData(resource.PostableResource):
    """Change notebook title ... could be more general.
    """

    def __init__(self, avatarId, kernelConnection, dbConnection):
        self.avatarId = avatarId
        self.kernelConnection = kernelConnection
        self.dbConnection = dbConnection

    def _changeOK(self, result):
        data = {'response':'ok', 'title':self.newtitle}
        jsondata = json.write(data, outputEncoding='utf-8')
        response = http.Response(responsecode.OK, CTYPE_JSON, jsondata)
        return response

    def _changeFailed(self, result):
        data = {'response':'ok'}
        jsondata = json.write(data, outputEncoding='utf-8')
        response = http.Response(responsecode.OK, CTYPE_JSON, jsondata)
        return response

    def render(self, req): 
        nbid = req.args['nbid'][0]
        self.newtitle = req.args['newtitle'][0]
        d = self.dbConnection.runOperation(sql.CHANGE_TITLE, (self.newtitle, nbid))
        d.addCallback(self._changeOK)
        return d


class TabCompleter(resource.PostableResource):
    def __init__(self, avatarId, kernelConnection, dbConnection):
        self.avatarId = avatarId
        self.kernelConnection = kernelConnection
        self.dbConnection = dbConnection
    
    def sendCodeToKernel(self, input):
        self.input = input
        if self.mode == 'name':
            self.output = self.kernelConnection.complete_name(self.input)
        else:
            self.output = self.kernelConnection.complete_attr(self.input)

        return self.output

    def returnResult(self, result):
        """Return result of some computation
        The last function in the chain of callbacks. 
        """
        self.output = str(result)
        data = {'completions':self.output, 'cellid':self.cellid}
        jsondata = json.write(data, outputEncoding='utf-8')
        response = http.Response(
            responsecode.OK, CTYPE_JSON, jsondata)
        return response

    def render(self, req): 
        """What initially handles the request.
        Get all the post args and go into the chain of defereds.
        """
        self.cellid = req.args['cellid'][0]
        self.nbid = req.args['nbid'][0]
        self.mode = req.args['mode'][0]
        s = req.args['input'][0]
        self.s = s.strip()
        # dt = threads.deferToThread(self.sendCodeToKernel, s)
        # dt.addCallback(self.returnResult)
        if self.mode == 'name':
            K = self.kernelConnection.complete_name(s)
        else:
            K = self.kernelConnection.complete_attr(s)
        K.addCallback(self.returnResult)
        return K

class SaveNotebookMetaData(resource.PostableResource):
    """Save non cell data from a notebook to the database.

    This is a resource used from an async call to '/bookshelf/save'
    to save data associated with a notebook.
    """
    def __init__(self, avatarId, dbConnection):
        self.avatarId = avatarId
        self.dbConnection = dbConnection

    def render(self, request):
        now = datetime.datetime.now()
        nbid = request.args['nbid'][0]
        orderlist = ','.join(request.args['orderlist'])
        cellsdata = json.read(request.args['cellsdata'][0])
        nbvalues = (orderlist, nbid)
        d = self.dbConnection.runOperation(sql.SAVE_NOTEBOOK_METADATA, nbvalues)
        def daisy_chain(res):
            """in place function for saving list of cells to database.
               consider replacing with non-deferred synchronous plain sqlite
               call.
            """
            try:
                cellid, celldata = cellsdata.popitem()
            except KeyError:
                return self._success

            de = self.dbConnection.runOperation(sql.SAVE_CELL, \
                (cellid,\
                now,\
                self.avatarId,\
                nbid,\
                celldata['content'],\
                celldata['cellstyle'],\
                celldata['props']))
            if len(cellsdata) > 0:
                de.addCallback(daisy_chain)
                return de
            else:
                de.addCallback(self._success)
                return de
        if (len(cellsdata) > 0):
            d.addCallback(daisy_chain)
        else:
            d.addCallback(self._success)
        return d

    def _success(self, result):
        jsondata = "{'response':'ok'}"
        response = http.Response(responsecode.OK, CTYPE_JSON, jsondata)
        return response



class KernelControl(resource.PostableResource):
    """Handle requests for controlling the kernel process.
    Two signals can be sent to the kernel
    Interrupt is supposed to stop the kernel from computing (if it is)
        - any pending eval requests will be aborted once the current one is
          interrupted
    Kill is supposed to completely take out the kernel process (if it is
    running) no matter what!
    """
    def __init__(self, avatarId, kernelConnection, dbConnection):
        self.avatarId = avatarId
        self.kernelConnection = kernelConnection
        self.dbConnection = dbConnection
 
    def _bad_request(self, result):
        html = "Bad process command. No such action"
        response = http.Response(responsecode.OK, CTYPE_UTF8, html)
        return response

    def _signalCallback(self, result):
        #if result:
        html = "done"
        html = str(result)
        response = http.Response(responsecode.OK, CTYPE_UTF8, html)
        return response

    def render(self, req):
        self.nbid = req.args['nbid'][0]
        action = req.args['action'][0].strip()
        
        actions = {'kill':self.kernelConnection.kill, 'interupt':self.kernelConnection.interrupt}
        if actions.has_key(action):
            d = defer.maybeDeferred(actions[action])
            d.addCallback(self._signalCallback)
        else:
            #FIXME: this might be bad code here! where is the callback
            # triggered?
            d = defer.Deferred()
            d.addCallback(self._bad_request)
        return d






