##################################################################### 
# Copyright (C) 2007 Alex Clemesha <clemesha@gmail.com>
# 
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
##################################################################### 
from knoboo.external.twisted.web2 import http_headers, responsecode
from knoboo.external.twisted.web2 import resource
from knoboo.external.twisted.web2 import http
from knoboo.external.twisted.web2 import server
from knoboo.external.twisted.web2 import static 
#standard library
import os
import time
import datetime
import md5

#twisted database acccess
from twisted.enterprise import adbapi
#json encoding
#import simplejson as json
from knoboo.external import minjson as json

#template for the html pages
from knoboo.templates import templates
#knoboo resources
from knoboo.resources import bookshelf
from knoboo.resources import notebook
from knoboo.resources import account
from knoboo.resources import settings

#sql static queries
import knoboo.sql as sql

import knoboo
k_path = knoboo.__path__[0]

#config
from knoboo import config
kbconfig = config.Config()
cfg = kbconfig.read_config()

CTYPE_UTF8 = {'content-type': http_headers.MimeType('text', 'html', {'charset':'utf-8'})}
CTYPE_JSON = {'content-type': http_headers.MimeType('application', 'json', {'charset':'utf-8'})}


class RegularUser(resource.PostableResource):
    """Top level resource of Knoboo.
    
    This resource always does a redirect to the
    users bookshelf so that after login (which is
    a POST operation), the postdata will not remain
    in the browser on return.  This method is sometimes
    referred to as the post-redirect-get method.
    """
    addSlash = True
    #child_static = static.File(os.path.join(os.path.abspath("."), "knoboo/static/"))
    child_static = static.File(os.path.join(k_path, "static/"))
    child_images = static.File(cfg['kernel_env'])

    def __init__(self, avatarId, cookie, kernelConnection, dbConnection): 
        self.cookie = cookie
        self.child_help = HelpPage()
        self.child_bookshelf = bookshelf.BookShelf(avatarId, dbConnection)    
        self.child_settings = settings.Settings(avatarId, dbConnection)
        self.child_notebook =  notebook.Notebook(avatarId, dbConnection)
        self.child_operation = notebook.NotebookOperations(avatarId, kernelConnection, dbConnection)    

    def render(self, request):
        response = http.RedirectResponse("/bookshelf")
        response.headers.setHeader("set-cookie", [http_headers.Cookie("sid", self.cookie)])
        return response


class LoginFailed(resource.PostableResource):
    addSlash = True
    def render(self, request):
        html = templates.index(error=True)         
        return http.Response(responsecode.OK, CTYPE_UTF8, html)

class AnonymousUser(resource.Resource):
    """The top level page as seen by an anonymous visitor.
    """
    addSlash = True
    child_signup = account.SignUp()
    child_login = LoginFailed()

    def __init__(self, cookie, dbConnection):
        #self.child_static = static.File(os.path.join(os.path.abspath("."), "knoboo/static/"))
        self.child_static = static.File(os.path.join(k_path, "static/"))
        self.child_create = account.CreateAccount(dbConnection)

    def locateChild(self, request, segments):
        if segments and segments[0] in ["logout", "bookshelf", "notebook", "settings"]:
            scheme, host = request.scheme, request.host
            return resource.RedirectResource(scheme=scheme, host=host, path="/"), ()
        return super(AnonymousUser, self).locateChild(request, segments)

    def render(self, request):
        html = templates.index()         
        return http.Response(responsecode.OK, CTYPE_UTF8, html)

class HelpPage(resource.Resource):
    """Help page"""

    def render(self, request):
        html = templates.help()         
        return http.Response(responsecode.OK, CTYPE_UTF8, html)

