##################################################################### 
# Copyright (C) 2007 Alex Clemesha <clemesha@gmail.com>
#                and Dorian Raymer <deldotdr@gmail.com>
# 
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
##################################################################### 

import os
import glob
from string import Template

import knoboo
k_path = knoboo.__path__[0]

#config
from knoboo import config
kbconfig = config.Config()
cfg = kbconfig.read_config()


def index(error=False, tmpl="templates/index.html"):
    msg = ''
    if error:
        msg = "<div class='error'>Login failed, please try again.</div>"
    d = {'error_message':msg}
    #fullpath = os.path.join(os.path.abspath("."), tmpl)
    fullpath = os.path.join(k_path, tmpl)
    return Template(open(fullpath).read()).substitute(d)


def signup(errors=None):
    d = {'un_require':"(<i>Username</i> must be at least 3 alpha-numeric characters)"}
    d.update({'pw_require':"(<i>Password</i> must be at least 5 alpha-numeric characters)"}) 
    #fullpath = os.path.join(os.path.abspath("."), "knoboo/templates/signup.html")
    fullpath = os.path.join(k_path, "knoboo/templates/signup.html")
    return Template(open(fullpath).read()).substitute(d)


def bookshelf(user):
    """Shows all of the users notebooks
    """
    compiled = glob.glob("knoboo/static/external/*-compiled-*") + glob.glob("knoboo/static/js/bookshelf-compiled-*")
    if compiled:
        the_scripts = ''
        for f in compiled:
            f = f.split("/")[-1]
            if f.startswith("external"):
                the_scripts += "<script type='text/javascript' src='/static/external/%s'></script>\n" % f
            else:
                the_scripts += "<script type='text/javascript' src='/static/js/%s'></script>\n" % f
    else: 
        the_scripts = script_link('/static/external/jquery.js')
        the_scripts += script_link('/static/external/jquery-ui.min.js')
        the_scripts += script_link('/static/external/jquery.color.js')
        the_scripts += script_link('/static/external/jquery.dom.js')
        the_scripts += script_link('/static/external/jquery.contextmenu.js')
        the_scripts += script_link('/static/external/jquery.curvycorners.packed.js')
        the_scripts += script_link('/static/external/jqModal.dev.js')
        the_scripts += script_link('/static/external/splitter.js')
        the_scripts += script_link('/static/js/bookshelf.js')

    d = {"script":the_scripts, "user":user}
    #fullpath = os.path.join(os.path.abspath("."), "knoboo/templates/bookshelf.html")
    fullpath = os.path.join(k_path, "templates/bookshelf.html")
    return Template(open(fullpath).read()).substitute(d)


def notebook(user, title, nbid, system):
    """The skeleton of a notebook.
    """
    compiled = glob.glob("knoboo/static/external/*-compiled-*") + glob.glob("knoboo/static/js/notebook-compiled-*")
    if compiled:
        the_scripts = ''
        for f in compiled:
            f = f.split("/")[-1]
            if f.startswith("external"):
                the_scripts += "<script type='text/javascript' src='/static/external/%s'></script>\n" % f
            else:
                the_scripts += "<script type='text/javascript' src='/static/js/%s'></script>\n" % f
        the_scripts += script("Notebook.nbid = '%s';"%nbid)
    else: 
        the_scripts = jquery()
        the_scripts += script_link('/static/js/Util.js')
        the_scripts += script_link('/static/js/BrowserDetect.js')
        the_scripts += script_link('/static/js/Notebook.js')
        the_scripts += script("Notebook.nbid = '%s';"%nbid)
        the_scripts += script_link('/static/js/Indicator.js')
        the_scripts += script_link('/static/js/Cell.js')
        the_scripts += script_link('/static/js/Async.js')
        the_scripts += script_link('/static/js/Delegator.js')
        the_scripts += script_link('/static/js/DOM.js')
        the_scripts += script_link('/static/js/SaveLoad.js')
        the_scripts += script_link('/static/js/Spawner.js')
        the_scripts += script_link('/static/js/TreeBranch.js')
        the_scripts += script_link('/static/js/Completer.js')
        the_scripts += script_link('/static/external/json2.js')
        the_scripts += script_link('/static/external/jquery.dimensions.js')
        the_scripts += script_link('/static/external/jquery.fieldselection.js')

    d = {"script":the_scripts, "title":title, "user":user}
    #fullpath = os.path.join(os.path.abspath("."), "knoboo/templates/notebook.html")
    fullpath = os.path.join(k_path, "templates/notebook.html")
    return Template(open(fullpath).read()).substitute(d)


def settings(user, back):
    """a users notebook settings page.
    """
    the_links = css_link('/static/styles/global.css')
    the_links = css_link('/static/style/style.css')
    the_links += css_link('/static/styles/settings.css')
    the_scripts = script_link('/static/js/jquery.js')

    d = {"style":the_links, "script":the_scripts, "user":user, "back":back}
    #fullpath = os.path.join(os.path.abspath("."), "knoboo/templates/settings.html")
    fullpath = os.path.join(k_path, "templates/settings.html")
    return Template(open(fullpath).read()).substitute(d)


def help():
    fullpath = os.path.join(k_path, "knoboo/templates/help.html")
    return open(fullpath).read()


###
# Helper functions
###
def script(script_code):
    """A block of javascript"""
    the_script = """
    <script type='text/javascript'> %s </script>
    """%script_code
    return the_script

def script_link(src):
    """Conviently make a javascript link"""
    return "<script src='%s' type='text/javascript'></script>\n"%(src)

def css_link(href):
    """Conviently make a css link"""
    return "<link rel='stylesheet' type='text/css' href='%s'></link>\n"%(href)

def jquery():
    if int(cfg['devel_mode']):
        scripts = script_link('/static/external/jquery.js')
    else:
        scripts = script_link('/static/external/jquery.js')
    scripts += script_link('/static/external/jquery.ifixpng.js');
    scripts += script_link('/static/external/jquery.contextmenu.js')
    scripts += script_link('/static/external/jquery.dom.js');
    return scripts
